package org.artifactory.repo.db;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.HaAddon;
import org.artifactory.addon.RestCoreAddon;
import org.artifactory.addon.xray.XrayAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.properties.PropertiesService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.repo.interceptor.StorageInterceptors;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.request.InternalRequestContext;
import org.artifactory.request.Request;
import org.artifactory.spring.InternalArtifactoryContext;
import org.artifactory.storage.fs.VfsItemProviderFactory;
import org.artifactory.storage.fs.VfsMutableFolderProvider;
import org.artifactory.storage.fs.service.FileService;
import org.artifactory.storage.fs.tree.VfsImmutableProvider;
import org.artifactory.storage.service.StatsServiceImpl;
import org.artifactory.test.ArtifactoryHomeBoundJUnitTest;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import static org.artifactory.descriptor.repo.SnapshotVersionBehavior.UNIQUE;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyBoolean;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SuppressWarnings({"rawtypes", "unchecked"})
class DbLocalRepoTest extends ArtifactoryHomeBoundJUnitTest {
    private AutoCloseable mocks;
    private DbLocalRepo localRepoSpy;

    @Mock private InternalArtifactoryContext context;
    @Mock private LocalRepoDescriptor descriptor;
    @Mock private InternalRepositoryService internalRepoService;
    @Mock private InternalRequestContext requestContext;
    @Mock private AddonsManager addonsManager;
    @Mock private AuthorizationService authorizationService;
    @Mock private StorageInterceptors storageInterceptors;
    @Mock private FileService fileService;
    @Mock private StatsServiceImpl statsService;
    @Mock private CentralConfigService centralConfigService;
    @Mock private PropertiesService propertiesService;
    @Mock private VfsItemProviderFactory vfsItemProviderFactory;
    @Mock private VfsMutableFolderProvider vfsMutableFolderProvider;
    @Mock private VfsImmutableProvider vfsImmutableProvider;
    @Mock private Request request;
    @Mock private XrayAddon xrayAddon;
    @Mock private RestCoreAddon restCoreAddon;
    @Mock private HaAddon haAddon;

    @BeforeEach
    void setUp() {
        mocks = MockitoAnnotations.openMocks(this);
        when(descriptor.getSnapshotVersionBehavior()).thenReturn(UNIQUE);
        when(descriptor.getKey()).thenReturn("repoKey");
        when(requestContext.getRequest()).thenReturn(request);
        when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        when(context.beanForType(AuthorizationService.class)).thenReturn(authorizationService);
        when(context.beanForType(StorageInterceptors.class)).thenReturn(storageInterceptors);
        when(context.beanForType(FileService.class)).thenReturn(fileService);
        when(context.beanForType(StatsServiceImpl.class)).thenReturn(statsService);
        when(context.beanForType(CentralConfigService.class)).thenReturn(centralConfigService);
        when(context.beanForType(InternalRepositoryService.class)).thenReturn(internalRepoService);
        when(context.beanForType(PropertiesService.class)).thenReturn(propertiesService);
        when(context.beanForType(VfsItemProviderFactory.class)).thenReturn(vfsItemProviderFactory);
        when(addonsManager.addonByType(XrayAddon.class)).thenReturn(xrayAddon);
        when(addonsManager.addonByType(RestCoreAddon.class)).thenReturn(restCoreAddon);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        when(restCoreAddon.getDynamicVersionContext(any(), any(), anyBoolean())).thenReturn(requestContext);
        when(vfsItemProviderFactory.createImmutableFileProvider(any(), any())).thenReturn(vfsImmutableProvider);
        when(context.getAuthorizationService()).thenReturn(authorizationService);
        when(vfsItemProviderFactory.createFolderProvider(any(), any(), any(), any()))
                .thenReturn(vfsMutableFolderProvider);
        ArtifactoryContextThreadBinder.bind(context);

        localRepoSpy = spy(new DbLocalRepo(descriptor, internalRepoService, null));
        localRepoSpy.init(false);
    }

    @AfterEach
    void tearDown() throws Exception {
        mocks.close();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testGetInfoPermissionsCheckNotCalledIfMavenAndNotFoundAndConstEnabled() {
        when(descriptor.getType()).thenReturn(RepoType.Maven);
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.enableMavenNonPreemptiveAuthentication.getPropertyName(),
                "true");
        localRepoSpy.getInfo(requestContext);
        verify(localRepoSpy, times(0)).checkDownloadIsAllowed(any());
    }

    @Test
    public void testGetInfoPermissionsCheckCalledIfNotMaven() {
        when(descriptor.getType()).thenReturn(RepoType.Docker);
        localRepoSpy.getInfo(requestContext);
        verify(localRepoSpy, times(1)).checkDownloadIsAllowed(any());
    }

    @Test
    public void testGetInfoPermissionsCheckCalledIfConstDisabled() {
        when(descriptor.getType()).thenReturn(RepoType.Maven);
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.enableMavenNonPreemptiveAuthentication.getPropertyName(),
                "false");
        localRepoSpy.getInfo(requestContext);
        verify(localRepoSpy, times(1)).checkDownloadIsAllowed(any());
    }
}