package org.artifactory.security.ssh;

import org.apache.sshd.server.session.ServerSession;
import org.artifactory.api.security.UserInfoBuilder;
import org.artifactory.storage.security.service.UserGroupStoreService;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.jfrog.security.util.BCProviderFactory;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentMatchers;
import org.mockito.Mockito;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.Security;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;

import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;

/**
 * @author Noam Y. Tenne
 */
class PublicKeyAuthenticatorTest {

    @Test
    void AuthenticatePublicKey() throws NoSuchAlgorithmException, InvalidKeySpecException {
        UserGroupStoreService userGroupStoreService = mock(UserGroupStoreService.class);
        ServerSession serverSession = mock(ServerSession.class);
        PublicKeyAuthenticator authenticator = new PublicKeyAuthenticator(userGroupStoreService);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA", BCProviderFactory.getProvider());
        PublicKey publicKey = keyFactory.generatePublic(new RSAPublicKeySpec(new BigInteger("1"), new BigInteger("3")));


        when(userGroupStoreService.findUserByProperty("sshPublicKey", "ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB", true))
                .thenReturn(new UserInfoBuilder("jim").build());
        boolean result = authenticator.authenticate("git", publicKey, serverSession);

        Assertions.assertTrue(result);
        verify(userGroupStoreService, times(1))
                .findUserByProperty("sshPublicKey", "ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB", true);
        verify(serverSession, times(1)).setAttribute(Mockito.any(), eq("jim"));
        verifyNoMoreInteractions(userGroupStoreService, serverSession);
    }

    @ParameterizedTest
    @ValueSource(ints = {256, 384, 521})
    void AuthenticateEcPublicKey(int num) throws NoSuchAlgorithmException, NoSuchProviderException {
        Security.addProvider(new BouncyCastleProvider());

        UserGroupStoreService userGroupStoreService = mock(UserGroupStoreService.class);
        ServerSession serverSession = mock(ServerSession.class);
        PublicKeyAuthenticator authenticator = new PublicKeyAuthenticator(userGroupStoreService);
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("EC", "BC");
        keyPairGenerator.initialize(num);
        java.security.KeyPair pair = keyPairGenerator.generateKeyPair();
        PublicKey publicKey = pair.getPublic();

        when(userGroupStoreService.findUserByProperty(eq("sshPublicKey"), ArgumentMatchers.matches(
                String.format("^ecdsa-sha2-nistp%s .+", num)), eq(true)))
                .thenReturn(new UserInfoBuilder("jim").build());
        boolean result = authenticator.authenticate("git", publicKey, serverSession);

        Assertions.assertTrue(result);
        verify(userGroupStoreService, times(1)).findUserByProperty(eq("sshPublicKey"), ArgumentMatchers.matches(
                String.format("^ecdsa-sha2-nistp%s .+", num)), eq(true));
        verify(serverSession, times(1)).setAttribute(Mockito.any(), eq("jim"));
        verifyNoMoreInteractions(userGroupStoreService, serverSession);
    }

    @Test
    void AuthenticatePublicKeyAndFindNoUser() throws NoSuchAlgorithmException, InvalidKeySpecException {
        UserGroupStoreService userGroupStoreService = mock(UserGroupStoreService.class);
        ServerSession serverSession = mock(ServerSession.class);
        PublicKeyAuthenticator authenticator = new PublicKeyAuthenticator(userGroupStoreService);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA", BCProviderFactory.getProvider());
        PublicKey publicKey = keyFactory.generatePublic(new RSAPublicKeySpec(new BigInteger("1"), new BigInteger("3")));

        when(userGroupStoreService.findUserByProperty("sshPublicKey", "ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB", true))
                .thenReturn(null);
        boolean result = authenticator.authenticate("git", publicKey, serverSession);

        Assertions.assertFalse(result);
        verify(userGroupStoreService, times(1))
                .findUserByProperty("sshPublicKey", "ssh-rsa AAAAB3NzaC1yc2EAAAABAwAAAAEB", true);
        verifyNoMoreInteractions(userGroupStoreService, serverSession);
    }
}