package org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.actions;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.xray.XrayAddon;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.model.common.RepoPathImpl;
import org.artifactory.projects.ProjectsService;
import org.artifactory.repo.RepoPath;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.ui.rest.model.artifacts.browse.treebrowser.nodes.actions.TreeNodeTabsPopulator.XRAY_TAB;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Yevdo Abramov
 * Created on 25/10/2020
 */
public class TreeNodeTabsPopulatorTest extends ArtifactoryHomeBoundTest {

    @Mock
    private ProjectsService projectsService;
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private XrayAddon xrayAddon;
    @Mock
    private AuthorizationService authorizationService;

    private TreeNodeTabsPopulator treeNodeTabsPopulator;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        treeNodeTabsPopulator = new TreeNodeTabsPopulator();
        treeNodeTabsPopulator.setProjectsService(projectsService);
        treeNodeTabsPopulator.setAddonsManager(addonsManager);
        treeNodeTabsPopulator.setAuthService(authorizationService);

        when(addonsManager.addonByType(XrayAddon.class)).thenReturn(xrayAddon);
    }

    @AfterMethod
    public void tearDown() {
        verifyNoMoreInteractions(projectsService, xrayAddon, authorizationService);
    }

    @Test
    public void testPopulateForRepositoryFromDefaultProject() {
        // given
        final String repoKey = "my_repo";
        TabsAndActions tabsAndActions = new TabsAndActions();
        tabsAndActions.setRepoPath(repoKey);
        tabsAndActions.setRepoType("local");
        when(projectsService.getProjectKeyOfRepository(repoKey)).thenReturn(ProjectsService.DEFAULT_PROJECT);
        when(authorizationService.canManage(any(RepoPath.class))).thenReturn(true);
        when(xrayAddon.isXrayConfPerRepoAndIntegrationEnabled(repoKey)).thenReturn(false);

        // when
        treeNodeTabsPopulator.populateForRepository(tabsAndActions);

        // then
        assertTrue(tabsAndActions.getTabs().stream()
                .anyMatch(tab -> tab.getName().equals(TreeNodeTabsPopulator.EFFECTIVE_PERMISSION_TAB)));
        verify(projectsService).getProjectKeyOfRepository(repoKey);
        verify(authorizationService, times(2)).canManage(any(RepoPath.class));
        verify(xrayAddon).isXrayConfPerRepoAndIntegrationEnabled(repoKey);
        verify(xrayAddon).isXrayEnabled();
    }

    @Test
    public void testPopulateForRepositoryFromProjectUserProject() {
        // given
        final String repoKey = "my_repo";
        TabsAndActions tabsAndActions = new TabsAndActions();
        tabsAndActions.setRepoPath(repoKey);
        tabsAndActions.setRepoType("local");
        when(projectsService.getProjectKeyOfRepository(repoKey)).thenReturn("user_project");
        when(authorizationService.canManage(any(RepoPath.class))).thenReturn(true);
        when(xrayAddon.isXrayConfPerRepoAndIntegrationEnabled(repoKey)).thenReturn(false);

        // when
        treeNodeTabsPopulator.populateForRepository(tabsAndActions);

        // then
        assertFalse(tabsAndActions.getTabs().stream()
                .anyMatch(tab -> tab.getName().equals(TreeNodeTabsPopulator.EFFECTIVE_PERMISSION_TAB)));
        verify(projectsService).getProjectKeyOfRepository(repoKey);
        verify(authorizationService).canManage(any(RepoPath.class));
        verify(xrayAddon).isXrayConfPerRepoAndIntegrationEnabled(repoKey);
        verify(xrayAddon).isXrayEnabled();
    }

    @Test(dataProvider = "testXrayDataProvider")
    public void testXray(boolean virtualSource, boolean xrayOfMinimalRequiredVersion, boolean xrayEnabled,
            boolean xrayConfPerRepoAndIntegrationEnabled, boolean handledByXray, String fileOrFolder,
            TabOrAction expected, int expectedCallsMinReqVer, int expectedCallsXrayEnabled,
            int expectedCallsConfPerRepoAndIntegEnabled, int expectedCallsHandledByXray) {
        RepoPathImpl repoPath = new RepoPathImpl("coolest-repo-ever", "path/to/file");
        when(xrayAddon.isXrayOfMinimalRequiredVersion(any()))
                .thenReturn(xrayOfMinimalRequiredVersion);
        when(xrayAddon.isXrayEnabled()).thenReturn(xrayEnabled);
        when(xrayAddon.isXrayConfPerRepoAndIntegrationEnabled(repoPath.getRepoKey()))
                .thenReturn(xrayConfPerRepoAndIntegrationEnabled);
        when(xrayAddon.isHandledByXray(repoPath)).thenReturn(handledByXray);
        TabsAndActions tabsAndActions = mock(TabsAndActions.class);
        when(tabsAndActions.getType()).thenReturn(fileOrFolder);
        TabOrAction tabOrAction = treeNodeTabsPopulator.xray(repoPath, virtualSource, tabsAndActions);
        assertEquals(tabOrAction, expected);
        verify(xrayAddon, times(expectedCallsMinReqVer)).isXrayOfMinimalRequiredVersion(any());
        verify(xrayAddon, times(expectedCallsXrayEnabled)).isXrayEnabled();
        verify(xrayAddon, times(expectedCallsConfPerRepoAndIntegEnabled))
                .isXrayConfPerRepoAndIntegrationEnabled(repoPath.getRepoKey());
        verify(xrayAddon, times(expectedCallsHandledByXray)).isHandledByXray(repoPath);
    }

    @DataProvider
    private Object[][] testXrayDataProvider() {
        TabOrAction tabOrAction = new TabOrAction(XRAY_TAB);
        return new Object[][] {
                {true, true, true, true, true, "file", null, 0, 0, 0, 0},
                {true, true, true, true, false, "file", null, 0, 0, 0, 0},
                {true, true, true, false, true, "file", null, 0, 0, 0, 0},
                {true, true, true, false, false, "file", null, 0, 0, 0, 0},
                {true, true, false, true, true, "file", null, 0, 0, 0, 0},
                {true, true, false, true, false, "file", null, 0, 0, 0, 0},
                {true, true, false, false, true, "file", null, 0, 0, 0, 0},
                {true, true, false, false, false, "file", null, 0, 0, 0, 0},
                {true, false, true, true, true, "file", null, 0, 0, 0, 0},
                {true, false, true, true, false, "file", null, 0, 0, 0, 0},
                {true, false, true, false, true, "file", null, 0, 0, 0, 0},
                {true, false, true, false, false, "file", null, 0, 0, 0, 0},
                {true, false, false, true, true, "file", null, 0, 0, 0, 0},
                {true, false, false, true, false, "file", null, 0, 0, 0, 0},
                {true, false, false, false, true, "file", null, 0, 0, 0, 0},
                {true, false, false, false, false, "file", null, 0, 0, 0, 0},
                {false, true, true, true, true, "file", tabOrAction, 1, 1, 0, 0},
                {false, true, true, true, false, "file", tabOrAction, 1, 1, 0, 0},
                {false, true, true, false, true, "file", tabOrAction, 1, 1, 0, 0},
                {false, true, true, false, false, "file", tabOrAction, 1, 1, 0, 0},
                {false, true, false, true, true, "file", tabOrAction, 0, 1, 1, 1},
                {false, true, false, true, false, "file", null, 0, 1, 1, 1},
                {false, true, false, false, true, "file", null, 0, 1, 1, 0},
                {false, true, false, false, false, "file", null, 0, 1, 1, 0},
                {false, false, true, true, true, "file", tabOrAction, 1, 1, 1, 1},
                {false, false, true, true, false, "file", null, 1, 1, 1, 1},
                {false, false, true, false, true, "file", null, 1, 1, 1, 0},
                {false, false, true, false, false, "file", null, 1, 1, 1, 0},
                {false, false, false, true, true, "file", tabOrAction, 0, 1, 1, 1},
                {false, false, false, true, false, "file", null, 0, 1, 1, 1},
                {false, false, false, false, true, "file", null, 0, 1, 1, 0},
                {false, false, false, false, false, "file", null, 0, 1, 1, 0},
                {true, true, true, true, true, "folder", null, 0, 0, 0, 0},
                {true, true, true, true, false, "folder", null, 0, 0, 0, 0},
                {true, true, true, false, true, "folder", null, 0, 0, 0, 0},
                {true, true, true, false, false, "folder", null, 0, 0, 0, 0},
                {true, true, false, true, true, "folder", null, 0, 0, 0, 0},
                {true, true, false, true, false, "folder", null, 0, 0, 0, 0},
                {true, true, false, false, true, "folder", null, 0, 0, 0, 0},
                {true, true, false, false, false, "folder", null, 0, 0, 0, 0},
                {true, false, true, true, true, "folder", null, 0, 0, 0, 0},
                {true, false, true, true, false, "folder", null, 0, 0, 0, 0},
                {true, false, true, false, true, "folder", null, 0, 0, 0, 0},
                {true, false, true, false, false, "folder", null, 0, 0, 0, 0},
                {true, false, false, true, true, "folder", null, 0, 0, 0, 0},
                {true, false, false, true, false, "folder", null, 0, 0, 0, 0},
                {true, false, false, false, true, "folder", null, 0, 0, 0, 0},
                {true, false, false, false, false, "folder", null, 0, 0, 0, 0},
                {false, true, true, true, true, "folder", null, 1, 1, 0, 0},
                {false, true, true, true, false, "folder", null, 1, 1, 0, 0},
                {false, true, true, false, true, "folder", null, 1, 1, 0, 0},
                {false, true, true, false, false, "folder", null, 1, 1, 0, 0},
                {false, true, false, true, true, "folder", tabOrAction, 0, 1, 1, 1},
                {false, true, false, true, false, "folder", null, 0, 1, 1, 1},
                {false, true, false, false, true, "folder", null, 0, 1, 1, 0},
                {false, true, false, false, false, "folder", null, 0, 1, 1, 0},
                {false, false, true, true, true, "folder", tabOrAction, 1, 1, 1, 1},
                {false, false, true, true, false, "folder", null, 1, 1, 1, 1},
                {false, false, true, false, true, "folder", null, 1, 1, 1, 0},
                {false, false, true, false, false, "folder", null, 1, 1, 1, 0},
                {false, false, false, true, true, "folder", tabOrAction, 0, 1, 1, 1},
                {false, false, false, true, false, "folder", null, 0, 1, 1, 1},
                {false, false, false, false, true, "folder", null, 0, 1, 1, 0},
                {false, false, false, false, false, "folder", null, 0, 1, 1, 0}
        };
    }
}
