/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.rest.resource.repositories;

import com.google.common.collect.Lists;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.rest.RestAddon;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.rest.constant.RepositoriesRestConstants;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RemoteRepoDescriptor;
import org.artifactory.descriptor.repo.RepoBaseDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.VirtualRepoDescriptor;
import org.artifactory.repo.RepoDetails;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.rest.common.util.RestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Nullable;
import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static javax.ws.rs.core.MediaType.APPLICATION_JSON;
import static javax.ws.rs.core.MediaType.TEXT_PLAIN;
import static org.apache.commons.lang.StringUtils.isNotBlank;
import static org.apache.http.HttpHeaders.CACHE_CONTROL;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.CACHE_CONTROL_NO_STORE;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.MT_FEDERATED_REPOSITORY_CONFIGURATION;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.MT_LOCAL_REPOSITORY_CONFIGURATION;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.MT_REMOTE_REPOSITORY_CONFIG;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.MT_REPOSITORY_DETAILS_LIST;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.MT_VIRTUAL_REPOSITORY_CONFIGURATION;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.PARAM_PACKAGE_TYPE;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.PARAM_REPO_TYPE;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.PATH_CONFIGURATION;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.PATH_ROOT;
import static org.artifactory.api.rest.constant.RepositoriesRestConstants.POSITION;
import static org.artifactory.descriptor.repo.RepoType.Generic;
import static org.artifactory.projects.ProjectsService.PROJECT;
import static org.artifactory.repo.RepoDetailsType.FEDERATED;
import static org.artifactory.repo.RepoDetailsType.LOCAL;
import static org.artifactory.repo.RepoDetailsType.REMOTE;
import static org.artifactory.repo.RepoDetailsType.VIRTUAL;
import static org.artifactory.repo.RepoDetailsType.valueOf;
import static org.artifactory.util.RepositoriesUtils.addNonRemoteRepoDetails;

/**
 * A resource to manage all repository related operations
 *
 * @author Noam Tenne
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
@Path(PATH_ROOT)
@RolesAllowed({AuthorizationService.ROLE_ADMIN, AuthorizationService.ROLE_USER})
public class RepositoriesResource {
    private static final Logger log = LoggerFactory.getLogger(RepositoriesResource.class);

    @Context
    HttpServletRequest httpRequest;

    @Context
    HttpServletResponse httpResponse;

    @Context
    private HttpHeaders requestHeaders;

    @Autowired
    private RepositoryService repositoryService;

    @Autowired
    private AuthorizationService authorizationService;

    @Autowired
    private AddonsManager addonsManager;

    /**
     * Repository configuration resource delegator
     *
     * @return RepoConfigurationResource
     */
    @Path("{repoKey}/" + RepositoriesRestConstants.PATH_CONFIGURATION)
    @Deprecated
    public RepoConfigurationResource getRepoConfigResource(@PathParam("repoKey") String repoKey) {
        return new RepoConfigurationResource(repositoryService, repoKey);
    }

    /**
     * Get repository configuration depending on the repository's type
     *
     * @param repoKey The repo Key
     * @return The repository configuration JSON
     */
    @GET
    @Path("{repoKey: .+}")
    @Produces({MT_LOCAL_REPOSITORY_CONFIGURATION, MT_REMOTE_REPOSITORY_CONFIG, MT_VIRTUAL_REPOSITORY_CONFIGURATION,
            MT_FEDERATED_REPOSITORY_CONFIGURATION, APPLICATION_JSON})
    public Response getRepoConfig(@PathParam("repoKey") String repoKey) {
        MediaType mediaType = requestHeaders.getMediaType();
        RestAddon restAddon = addonsManager.addonByType(RestAddon.class);
        return restAddon.getRepositoryConfiguration(repoKey, mediaType);
    }

    @PUT
    @Consumes({MT_LOCAL_REPOSITORY_CONFIGURATION, MT_REMOTE_REPOSITORY_CONFIG, MT_VIRTUAL_REPOSITORY_CONFIGURATION,
            MT_FEDERATED_REPOSITORY_CONFIGURATION, APPLICATION_JSON})
    @Produces(TEXT_PLAIN)
    @Path("{repoKey: .+}")
    public Response createOrReplaceRepository(@PathParam("repoKey") String repoKey,
            @QueryParam(POSITION) int position, Map repositoryConfiguration) {
        MediaType mediaType = requestHeaders.getMediaType();
        RestAddon restAddon = addonsManager.addonByType(RestAddon.class);
        return restAddon.createOrReplaceRepository(repoKey, repositoryConfiguration, mediaType, position);
    }

    @POST
    @Consumes({MT_LOCAL_REPOSITORY_CONFIGURATION, MT_REMOTE_REPOSITORY_CONFIG, MT_VIRTUAL_REPOSITORY_CONFIGURATION,
            MT_FEDERATED_REPOSITORY_CONFIGURATION, APPLICATION_JSON})
    @Produces(TEXT_PLAIN)
    @Path("{repoKey: .+}")
    public Response updateRepository(@PathParam("repoKey") String repoKey, Map repositoryConfiguration) {
        MediaType mediaType = requestHeaders.getMediaType();
        RestAddon restAddon = addonsManager.addonByType(RestAddon.class);
        return restAddon.updateRepository(repoKey, repositoryConfiguration, mediaType);
    }

    /**
     * Returns a JSON list of repository details.
     * <p/>
     * NOTE: Used by CI integration to get a list of deployment repository targets.
     *
     * @param repoType Name of repository type, as defined in {@link org.artifactory.repo.RepoDetailsType}. Can be null
     * @param projectKey If supplied then we shall return only repositories mapped to this particular project
     * @return JSON repository details list. Will return details of defined type, if given. And will return details of
     * all types if not
     */
    @GET
    @Produces({MT_REPOSITORY_DETAILS_LIST, APPLICATION_JSON})
    public Response getAllRepoDetails(@QueryParam(PARAM_REPO_TYPE) String repoType,
            @QueryParam(PARAM_PACKAGE_TYPE) String packageType,
            @QueryParam(PROJECT) String projectKey) {
        log.debug("Fetching all repositories of type {}, package {}, project {}", repoType, packageType, projectKey);
        List<RepoDetails> repoDetailsList = getRepoDetailsList(repoType, packageType, projectKey);
        return Response.ok().entity(repoDetailsList).header(CACHE_CONTROL, CACHE_CONTROL_NO_STORE).build();
    }

    @DELETE
    @Produces(TEXT_PLAIN)
    @Path("{repoKey: .+}")
    public Response deleteRepository(@PathParam("repoKey") String repoKey) {
        RestAddon restAddon = addonsManager.addonByType(RestAddon.class);
        return restAddon.deleteRepository(repoKey);
    }

    /**
     * Returns a list of repository details
     *
     * @param repoType Name of repository type, as defined in {@link org.artifactory.repo.RepoDetailsType}
     * @return List of repository details
     */
    private List<RepoDetails> getRepoDetailsList(@Nullable String repoType, @Nullable String packageType,
            @Nullable String projectKey) {
        RepoDetailsType selectedType = null;
        if (isNotBlank(repoType)) {
            try {
                selectedType = valueOf(repoType.toUpperCase());
            } catch (IllegalArgumentException e) {
                return Lists.newArrayList();
            }
        }
        RepoType selectedPackageType = null;
        if (isNotBlank(packageType)) {
            selectedPackageType = RepoType.fromType(packageType);
            //RepoType.fromType(packageType) returns "Generic" when the input does not match the enum, and we
            //don't want to return all generic repos on user error.
            if (Generic.equals(selectedPackageType) && !Generic.name().equalsIgnoreCase(packageType)) {
                return Lists.newArrayList();
            }
        }
        return getReposList(selectedType, selectedPackageType, projectKey);
    }

    private List<RepoDetails> getReposList(@Nullable RepoDetailsType selectedType, @Nullable RepoType packageType,
            @Nullable String projectKey) {
        List<RepoDetails> detailsList = Lists.newArrayList();
        String contextUrl = RestUtils.getServletContextUrl(httpRequest);
        if (selectedType == null || LOCAL.equals(selectedType)) {
            List<LocalRepoDescriptor> localRepoDescriptorList = filterByPackageType(
                    repositoryService.getLocalRepoDescriptorsByProject(projectKey), packageType);
            addNonRemoteRepoDetails(contextUrl, detailsList, localRepoDescriptorList, LOCAL, authorizationService);
        }
        if (selectedType == null || FEDERATED.equals(selectedType)) {
            List<FederatedRepoDescriptor> federatedRepoDescriptorList = filterByPackageType(
                    repositoryService.getFederatedRepoDescriptorsByProject(projectKey), packageType);
            addNonRemoteRepoDetails(contextUrl, detailsList, federatedRepoDescriptorList, FEDERATED,
                    authorizationService);
        }
        if (selectedType == null || REMOTE.equals(selectedType)) {
            addRemoteRepoDetails(detailsList, packageType, projectKey);
        }
        if (selectedType == null || VIRTUAL.equals(selectedType)) {
            List<VirtualRepoDescriptor> remoteRepoDescriptorList = filterByPackageType(
                    repositoryService.getVirtualRepoDescriptorsByProject(projectKey), packageType);
            addNonRemoteRepoDetails(contextUrl, detailsList, remoteRepoDescriptorList, VIRTUAL, authorizationService);
        }
        return detailsList;
    }

    /**
     * Adds a list of remote repositories to the repo details list
     *
     * @param detailsList List that details should be appended to
     */
    private void addRemoteRepoDetails(List<RepoDetails> detailsList, RepoType packageType,
            @Nullable String projectKey) {
        List<RemoteRepoDescriptor> remoteRepos =
                filterByPackageType(repositoryService.getRemoteRepoDescriptorsByProject(projectKey), packageType);

        for (RemoteRepoDescriptor remoteRepo : remoteRepos) {
            String key = remoteRepo.getKey();
            if (authorizationService.userHasPermissionsOnRepositoryRoot(key)) {
                String configUrl = null;
                if (remoteRepo.isShareConfiguration()) {
                    configUrl = getRepoConfigUrl(key);
                }
                detailsList.add(new RepoDetails(key, remoteRepo.getDescription(), REMOTE, remoteRepo.getUrl(),
                        configUrl, remoteRepo.getType()));
            }
        }
    }

    private <T extends RepoBaseDescriptor> List<T> filterByPackageType(List<T> repos, @Nullable RepoType packageType) {
        //No type - return all requested repos not filtered by any type
        if (packageType == null) {
            return repos;
        }
        return repos.stream()
                .filter(repo -> packageType.equals(repo.getType()))
                .collect(Collectors.toList());
    }

    /**
     * Returns the repository configuration URL
     *
     * @param repoKey Key of repository to assemble URL for
     * @return Repository configuration URL
     */
    private String getRepoConfigUrl(String repoKey) {
        return RestUtils.getRestApiUrl(httpRequest) + "/" + PATH_ROOT + "/" + repoKey + "/" + PATH_CONFIGURATION;
    }
}
