package org.artifactory.rest.resource.repositories;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.KeyPairDescriptor;
import org.artifactory.descriptor.repo.RepoBaseDescriptor;
import org.jfrog.security.common.AuthorizationRoles;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.Objects;

import static javax.ws.rs.core.Response.Status.BAD_REQUEST;

/**
 * A resource to manage all repository key pairs
 * @author Omri Ziv
 */
@RolesAllowed({AuthorizationRoles.ROLE_ADMIN, AuthorizationRoles.ROLE_USER})
public class RepositoryKeyPairsResource {

    private static final String KEY_PAIR_DOES_NOT_EXISTS_MSG = "Key Pair does not exist";
    private CentralConfigService centralConfigService;
    private String repoKey;

    public RepositoryKeyPairsResource(CentralConfigService centralConfigService,
            String repoKey) {
        this.centralConfigService = centralConfigService;
        this.repoKey = repoKey;
    }

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    @Path("promote")
    public Response promote() {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        if (Objects.isNull(repoDescriptor.getSecondaryKeyPairDescriptor())) {
            return Response.status(BAD_REQUEST)
                    .entity("Nothing to promote")
                    .build();
        }
        repoDescriptor.promoteKeyPair();
        centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
        return Response.ok()
                .entity("Secondary Key Pair Of repo '" + repoKey + "' promoted.")
                .build();
    }

    @POST
    @Consumes(MediaType.TEXT_PLAIN)
    @Produces(MediaType.APPLICATION_JSON)
    public Response setKey(String keyName) {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPairDescriptor = configDescriptor.getKeyPair(keyName);
        if (Objects.isNull(keyPairDescriptor)) {
            return Response.status(BAD_REQUEST)
                    .entity(KEY_PAIR_DOES_NOT_EXISTS_MSG)
                    .build();
        }
        if (keyPairDescriptor.equals(repoDescriptor.getPrimaryKeyPairDescriptor())) {
            return Response.ok()
                    .entity(String.format("Key Pair '%s' of repo '%s' was set as primary.", keyName, repoKey))
                    .build();
        }
        if (Objects.isNull(repoDescriptor.getPrimaryKeyPairDescriptor())) {
            repoDescriptor.setPrimaryKeyPairDescriptor(keyPairDescriptor);
            centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
            return Response.ok()
                    .entity(String.format("Key Pair '%s' of repo '%s' was set as primary.", keyName, repoKey))
                    .build();
        } else {
            repoDescriptor.setSecondaryKeyPairDescriptor(keyPairDescriptor);
            centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
            return Response.ok()
                    .entity(String.format("Key Pair '%s' of repo '%s' was set as secondary.", keyName, repoKey))
                    .build();
        }
    }

    @DELETE
    @Produces(MediaType.APPLICATION_JSON)
    @Path("{keyName: .+}")
    public Response deleteKey(@PathParam("keyName") String keyName) {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPairDescriptor = configDescriptor.getKeyPair(keyName);
        if (Objects.isNull(keyPairDescriptor)) {
            return Response.status(BAD_REQUEST)
                    .entity(KEY_PAIR_DOES_NOT_EXISTS_MSG)
                    .build();
        }
        if (keyPairDescriptor.equals(repoDescriptor.getPrimaryKeyPairDescriptor())) {
            repoDescriptor.setPrimaryKeyPairDescriptor(null);
            repoDescriptor.promoteKeyPair();
            centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
            return Response.ok()
                    .entity(String.format("Primary Key Pair '%s' of repo '%s' removed.", keyName, repoKey))
                    .build();

        } else if (keyPairDescriptor.equals(repoDescriptor.getSecondaryKeyPairDescriptor())) {
            repoDescriptor.setSecondaryKeyPairDescriptor(null);
            centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
            return Response.ok()
                    .entity(String.format("Secondary Key Pair '%s' of repo '%s' removed.", keyName, repoKey))
                    .build();
        } else {
            return Response.status(BAD_REQUEST)
                    .entity("Repo '" + repoKey + "' Not containing Key Pair '" + keyName + "'")
                    .build();
        }
    }

    @POST
    @Consumes(MediaType.TEXT_PLAIN)
    @Produces(MediaType.APPLICATION_JSON)
    @Path("primary")
    public Response setPrimary(String primaryKeyName) {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPairDescriptor = configDescriptor.getKeyPair(primaryKeyName);
        if (Objects.isNull(keyPairDescriptor)) {
            return Response.status(BAD_REQUEST)
                    .entity(KEY_PAIR_DOES_NOT_EXISTS_MSG)
                    .build();
        }
        repoDescriptor.setPrimaryKeyPairDescriptor(keyPairDescriptor);
        centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
        return Response.ok()
                .entity("Key Pair '" + primaryKeyName + "' of repo '" + repoKey + "' was set as primary.")
                .build();
    }

    @DELETE
    @Produces(MediaType.APPLICATION_JSON)
    @Path("primary")
    public Response deletePrimary() {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        repoDescriptor.setPrimaryKeyPairDescriptor(null);
        repoDescriptor.promoteKeyPair();
        centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
        return Response.ok()
                .entity("Primary Key Pair of repo '" + repoKey + "' deleted successfully.")
                .build();
    }


    @POST
    @Consumes(MediaType.TEXT_PLAIN)
    @Produces(MediaType.APPLICATION_JSON)
    @Path("secondary")
    public Response setSecondary(String secondaryKeyName) {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPairDescriptor = configDescriptor.getKeyPair(secondaryKeyName);
        if (Objects.isNull(keyPairDescriptor)) {
            return Response.status(BAD_REQUEST)
                    .entity(KEY_PAIR_DOES_NOT_EXISTS_MSG)
                    .build();
        }
        if (Objects.isNull(repoDescriptor.getPrimaryKeyPairDescriptor())) {
            return Response.status(BAD_REQUEST)
                    .entity("Could not set secondary while primary is not set")
                    .build();
        }

        repoDescriptor.setSecondaryKeyPairDescriptor(keyPairDescriptor);
        centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
        return Response.ok()
                .entity("Key Pair '" + secondaryKeyName + "' of repo '" + repoKey + "' was set as secondary.")
                .build();
    }

    @DELETE
    @Produces(MediaType.APPLICATION_JSON)
    @Path("secondary")
    public Response deleteSecondary() {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        repoDescriptor.setSecondaryKeyPairDescriptor(null);
        centralConfigService.saveEditedDescriptorAndReload(configDescriptor, repoDescriptor);
        return Response.ok()
                .entity("Secondary Key Pair of repo '" + repoKey + "' deleted successfully.")
                .build();
    }

    @GET
    @Path("primary/public")
    @Produces(MediaType.TEXT_PLAIN)
    @RolesAllowed({AuthorizationService.ROLE_USER, AuthorizationService.ROLE_ADMIN})
    public Response getPrimaryPublicKey() {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPair = repoDescriptor.getPrimaryKeyPairDescriptor();
        if (Objects.isNull(keyPair)) {
            return Response.status(BAD_REQUEST)
                    .entity("Repo '" + repoKey + "' Not containing primary Key Pair")
                    .build();

        }
        return Response.ok()
                .header("Content-Disposition",
                        "attachment; filename=" + keyPair.getAliasWithExtension())
                .entity(keyPair.getPublicKey()).build();
    }

    @GET
    @Path("secondary/public")
    @Produces(MediaType.TEXT_PLAIN)
    @RolesAllowed({AuthorizationService.ROLE_USER, AuthorizationService.ROLE_ADMIN})
    public Response getSecondaryPublicKey() {
        MutableCentralConfigDescriptor configDescriptor = centralConfigService.getMutableDescriptor();
        RepoBaseDescriptor repoDescriptor = configDescriptor.findRepoDescriptor(repoKey);
        KeyPairDescriptor keyPair = repoDescriptor.getSecondaryKeyPairDescriptor();
        if (Objects.isNull(keyPair)) {
            return Response.status(BAD_REQUEST)
                    .entity("Repo '" + repoKey + "' Not containing primary Key Pair")
                    .build();

        }
        return Response.ok()
                .header("Content-Disposition",
                        "attachment; filename=" + keyPair.getAliasWithExtension())
                .entity(keyPair.getPublicKey()).build();
    }


}
